/*
 * File descriptor shared memory functions.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>

#include "fsdesc.h"
#include "fsops.h"

/* Copy data from the given buffer to the start of the descriptor's data,
   updating the data position. */

void fs_write_desc(file_descriptor_t *desc, char *buf, size_t to_transfer)
{
  memcpy(fs_get_data_start(desc), buf, to_transfer);

  /* Update position details. */

  desc->data_start += to_transfer;

  if (desc->data_start > desc->data_end)
    desc->data_end = desc->data_start;

  desc->written = 1;
}

/* Copy data to the given buffer from the start of the descriptor's data,
   updating the data position. */

void fs_read_desc(file_descriptor_t *desc, char *buf, size_t to_transfer)
{
  memcpy(buf, fs_get_data_start(desc), to_transfer);

  /* Update position details. */

  desc->data_start += to_transfer;
}

/* Return the amount of available data. */

size_t fs_get_available(file_descriptor_t *desc)
{
  if (desc->data_end >= desc->data_start)
    return desc->data_end - desc->data_start;
  else
    return 0;
}

/* Return a pointer to the available data. */

char *fs_get_data_start(file_descriptor_t *desc)
{
  return desc->obj.buffer.addr + desc->data_start;
}

/* Return the position in the file of the start of available data. */

size_t fs_get_data_start_position(file_descriptor_t *desc)
{
  return desc->obj.start_pos + desc->data_start;
}

/* Return a pointer to the end of available data. */

char *fs_get_data_end(file_descriptor_t *desc)
{
  return desc->obj.buffer.addr + desc->data_end;
}

/* Return the position in the file of the end of available data. */

size_t fs_get_data_end_position(file_descriptor_t *desc)
{
  return desc->obj.start_pos + desc->data_end;
}

/* Return the amount of space remaining in the buffer. */

size_t fs_get_space(file_descriptor_t *desc)
{
  return desc->obj.buffer.size - desc->data_start;
}

/* Return whether the given position is provided by the buffer. */

int fs_have_position(file_descriptor_t *desc, size_t position)
{
  return (position >= desc->obj.start_pos) &&
         (position < fs_get_data_end_position(desc));
}

/* Return whether there is no available data to read. */

int fs_no_available_data(file_descriptor_t *desc)
{
  return !desc->data_end;
}

/* Update the end of data position given the new size of the object. */

void fs_update_from_size(file_descriptor_t *desc)
{
  if (desc->obj.size > fs_get_data_end_position(desc))
  {
    size_t extent = desc->obj.size - desc->obj.start_pos;

    if (extent > desc->obj.buffer.size)
      extent = desc->obj.buffer.size;

    desc->data_end = extent;
  }
}
